/*
 * This file is part of pygal
 *
 * A python svg graph plotting library
 * Copyright © 2012 Kozea

 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with pygal. If not, see <http://www.gnu.org/licenses/>.
*/

svg {
  background-color: {{ style.background }};
}

svg * {
  -webkit-transition: {{ style.transition }};
  -moz-transition: {{ style.transition }};
  transition: {{ style.transition }};
}

.graph > .background {
  fill: {{ style.background }};
}

.plot > .background {
  fill: {{ style.plot_background }};
}

.graph {
  fill: {{ style.foreground }};
}

text.no_data {
  fill: {{ style.foreground_light }};
  text-anchor: middle;
}

.line {
  fill-opacity: 0;
}

.centered {
  text-anchor: middle;
}

.title {
  fill: {{ style.foreground_light }};
  font-size: {{ font_sizes.title }};
  text-anchor: middle;
}

.legends .legend text {
  font-family: monospace;
  font-size: {{ font_sizes.legend }};
  fill: {{ style.foreground }};
  fill-opacity: 1;
}

.legends .legend:hover text {
  fill: {{ style.foreground_light }};
}

.axis text {
  font-size: {{ font_sizes.label }};
  font-family: sans;
}

.axis.x text {
  font-family: monospace;
  text-anchor: middle;
}

.axis.x:not(.web) text[transform] {
  text-anchor: start;
}

.axis.y text {
  font-family: monospace;
  text-anchor: end;
}

.axis.y .logarithmic text:not(.major) {
  font-size: 50%;
}

.axis .line {
  stroke: {{ style.foreground_light }};
}

.axis .guide.line {
  stroke: {{ style.foreground_dark }};
  stroke-dasharray: 4,4;
}

.axis .major.line {
  stroke: {{ style.foreground }};
  stroke-dasharray: 6,6;
}
.axis text.major {
  stroke-width: 0.5px;
  stroke: {{ style.foreground_light }};
  fill: {{ style.foreground_light }};
}

.axis.{{ hidden }} .guide.line {
  opacity: 0;
}

.axis.y .guides:hover .guide.line,
.line-graph .axis.x .guides:hover .guide.line,
.stackedline-graph .axis.x .guides:hover .guide.line,
.xy-graph .axis.x .guides:hover .guide.line {
  stroke: {{ style.foreground_light }};
  opacity: 1;
}

.axis .guides:hover text {
  fill: {{ style.foreground_light }};
  opacity: 1;
}

.nofill {
  fill: none;
}

.reactive {
  fill-opacity: {{ style.opacity }};
}

.reactive.active, .active .reactive {
  fill-opacity:  {{ style.opacity_hover }};
}

.dot {
  stroke-width: 1px;
  fill-opacity: 1;
}

.dot.active {
  stroke-width: 5px;
}

.series text {
  font-size: {{ font_sizes.value }};
  stroke: none;
  fill: {{ style.foreground_light }};
}

.series text.active {
  opacity: 1;
}

#tooltip rect {
  fill-opacity: 0.8;
  fill: {{ style.plot_background }};
  stroke: {{ style.foreground_light }};
}

#tooltip text {
  fill-opacity: 1;
  fill: {{ style.foreground_light }};
  font-size: {{ font_sizes.tooltip }};
}

{{ style.colors }}
